var RDF = Components
    .classes['@mozilla.org/rdf/rdf-service;1']
    .getService()
    .QueryInterface(Components.interfaces.nsIRDFService);
var RDFC = Components
    .classes["@mozilla.org/rdf/container-utils;1"]
    .getService(Components.interfaces.nsIRDFContainerUtils);

const URL = new Components.Constructor("@mozilla.org/network/standard-url;1", "nsIURL");
const RDF_Seq        = RDF.GetResource( "http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq" );
const RDF_type       = RDF.GetResource( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
const RDFS_seeAlso   = RDF.GetResource( "http://www.w3.org/2000/01/rdf-schema#seeAlso" );
const DC_description = RDF.GetResource( "http://purl.org/dc/elements/1.1/description" );
const DC_title       = RDF.GetResource( "http://purl.org/dc/elements/1.1/title" );
const REF_object     = RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#object" );
const REF_method     = RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#method" );
const REF_property   = RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#property" );
const MOZREF_searchRoot = RDF.GetResource("mozref:search");

var gContentsTree;
var gDescription;
var gLastQuery;

function MozRefSidebarInit() {
    gContentsTree = document.getElementById("mozrefContentsTree");
    gDescription = document.getElementById("mozrefDescription");

    gContentsTree.setAttribute('datasources', mozrefService.getUrl() + "?style=rdf");
    gDescription.setAttribute('datasources', mozrefService.getUrl() + "?style=rdf");

    makeSearchRoot();
    emptySearchResults();
    gLastQuery = "";
    searchContents("");
}

function onOpenNode(event) {
    var uri = getSelectedURI();
    if (uri == null) return null;

    var source = RDF
        .GetResource(uri)
        .QueryInterface(Components.interfaces.nsIRDFNode);

    var url = gContentsTree
        .database
        .GetTarget(source, RDFS_seeAlso, true)
        .QueryInterface(Components.interfaces.nsIRDFResource)
        .Value;

    switch (mozrefService.getOpenAction()) {
        case "newWindow":
            openNewWindowWith(url, null, false);
            break;
        case "newTab":
            openNewTabWith(url, null, event, false);
            break;
        case "none":
            break;
        case "currWindow":
            // Fall-through
        default:
            openTopWin(url);
            break;
    }
}

function onSelectNode(event) {
    var uri = getSelectedURI();
    gDescription.value = "";
    if (uri == null)
        return null;

    var source = RDF
        .GetResource(uri)
        .QueryInterface(Components.interfaces.nsIRDFNode);

    var description = gContentsTree
        .database
        .GetTarget(source, DC_description, true)
        .QueryInterface(Components.interfaces.nsIRDFLiteral)
        .Value;

    // Set the RDF context for the bottom text box
    gDescription.value = description;
}

function getSelectedURI() {
    var currentIndex = gContentsTree.view.selection.currentIndex;
    if (currentIndex < 0) return null;
    return gContentsTree.view.getResourceAtIndex(currentIndex).Value;
}

function searchContents(searchText) {
    if (searchText == "") {
        gContentsTree.setAttribute('ref', mozrefService.getUrl());
        gLastQuery = "";
        return;
    }
    gContentsTree.setAttribute('ref', MOZREF_searchRoot.Value);

    // Determine how we are going to change an existing search criteria
    if (gLastQuery == "") {
        // This is a new search, just do it.
        emptySearchResults();
        makeSearchResults(searchText);
    } else {
        // We're modifying a search; check to see how it should be done
        if (gLastQuery.length < searchText.length && searchText.indexOf(gLastQuery) == 0) {
            // Handle when an extra character is added to an existing search.
            refineSearchResults(searchText);
        } else if (gLastQuery.length > searchText.length && gLastQuery.indexOf(searchText) == 0) {
            // Handle when a character has been removed
            makeSearchResults(searchText);
        } else {
            // Bah, its a new search...run it.
            emptySearchResults();
            makeSearchResults(searchText);
        }
    }
    gLastQuery = searchText;
    gContentsTree.builder.rebuild();
}


function refineSearchResults(query) {
    query = query.toLowerCase();
    var ds = gContentsTree.database;
    var container = getSearchRoot();

    var removeArray = new Array();
    var children = container.GetElements();
    while (children.hasMoreElements()) {
        var child = children.getNext();
        if (child instanceof Components.interfaces.nsIRDFResource) {
            child = child.QueryInterface(Components.interfaces.nsIRDFResource);

            if (child.Value.toLowerCase().indexOf(query) == -1) {
                removeArray.push(child);
            }
        }
    }

    for (var idx = 0; idx < removeArray.length; idx++) {
        container.RemoveElement(removeArray[idx], true);
    }
}

function makeSearchResults(query) {
    query = query.toLowerCase();
    var ds = gContentsTree.database;
    var predicates = new Array(REF_object, REF_method, REF_property);
    var container = getSearchRoot();

    for (var idx in predicates) {
        var targets = ds.GetSources(RDF_type, predicates[idx], true);
        while (targets.hasMoreElements()) {
            var uri = targets
                .getNext()
                .QueryInterface(Components.interfaces.nsIRDFResource);

            var title = ds.GetTarget(uri, DC_title, true);
            if (title instanceof Components.interfaces.nsIRDFLiteral) {
                title = title.QueryInterface(Components.interfaces.nsIRDFLiteral);
            } else {
                continue;
            }

            if (title.Value.toLowerCase().indexOf(query) > -1) {
                // Skip if we should not add this again
                if (RDFC.indexOf(ds, MOZREF_searchRoot, uri) > -1) {
                    continue;
                }

                try {
                    container.AppendElement(uri);
                } catch (e) {
                    dump("Aack, we had an exception " + e + "\n");
                }
            }
        }
    }
}

function emptySearchResults() {
    var container = getSearchRoot();

    // Loop through the children, and remove them
    var results = container.GetElements();
    var idx = 0;
    while (results.hasMoreElements()) {
        results.getNext();
        idx++;
    }
    if (idx == 0)
        return;

    while (idx > 0) {
        //var renumberList = idx == 1 ? true : false;
        try {
            container.RemoveElementAt(idx, true);
        } catch (e) {
            dump("Deleting " + idx + " failed: " + e + "\n");
        }
        idx--;
    }
    emptySearchResults(query);
}

function getSearchRoot() {
    var ds = gContentsTree.database;

    try {
        var container = Components.classes["@mozilla.org/rdf/container;1"].
                          createInstance(Components.interfaces.nsIRDFContainer);
        container.Init(ds, MOZREF_searchRoot);
        return container
    } catch (e) {
        // Don't do anything, since this resource doesn't exist
        // This shouldn't happen, since we create the container
        // above, but you never know.
        return null;
    }
}

function makeSearchRoot() {
    var ds = gContentsTree.database;

    var container = RDFC.MakeSeq(ds, MOZREF_searchRoot);
}
